# -*- coding: utf-8 -*-
"""
Created on Thu Jun 16 11:51:03 2016

@author: herranz
"""

import numpy as np
import random_genlaplace as rgen

def log_prior_podegree(poldeg):

    polmean = np.array([2.01,1.76,1.71])
    C       = np.matrix([[0.65,0.30,0.18],[0.30,0.48,0.30],[0.18,0.30,0.45]])
    Cinv    = np.linalg.inv(C)
    y       = np.matrix(np.where(poldeg<0,-np.inf,np.log(np.divide(poldeg,polmean))))
    cuerpo  = -float(y*Cinv*np.transpose(y))/2.0

    normy   = ((2*np.pi)**3)/(np.prod(poldeg)*np.abs(np.linalg.det(C)))
    if normy<0:
        lognormalizacion = -np.inf
    else:
        lognormalizacion = np.log(normy)

    return cuerpo+lognormalizacion

def log_prior_angle(angle):
    if (angle>=0)&(angle<np.pi):
        y = np.log(1.0/np.pi)
    else:
        y = -np.inf
    return y

def log_prior_rot(rot):
    scale    = 0.00477 #rad
    mean_rot = 0.00256 #rad
    y        = np.log(rgen.genlaplace(rot,mean_rot,scale,0.5))
#    y        = np.log(b/2.0) - b * np.abs(rot_arcmin-mean_rot)
    return y

def log_prior(parametros):
    p1,p2,p3,rot,psi = parametros
    y = log_prior_podegree(np.array([p1,p2,p3])) + log_prior_rot(rot) + log_prior_angle(psi)
    return y


def log_likelihood(parametros,datos,size,intensidades,freqs,fwhms,nlevels):

    import pol_simulations as sim
    template = sim.make_templates(parametros,size,intensidades,freqs,fwhms,toplot=False)
    indices = [0,1,2]
    s = 0.0

    for i in indices:
        m1 = (datos[:,:,2*i]-template[:,:,2*i])**2 / nlevels[i]*2
        m2 = (datos[:,:,2*i+1]-template[:,:,2*i+1])**2 / nlevels[i]*2
        s  = s + np.sum(m1)+np.sum(m2)

    s = -0.5*s

    return s

def log_posterior(parametros,datos,size,intensidades,freqs,fwhms,nlevels):
    p1,p2,p3,rot,psi = parametros
    a = log_prior(parametros)
    if np.isnan(a):
        a = -np.inf
    b = log_likelihood(parametros,datos,size,intensidades,freqs,fwhms,nlevels)
    if np.abs(rot)>(0.5):
        b = -np.inf
    return a+b

def log_posterior_fake(parametros,datos,size,intensidades,freqs,fwhms,nlevels):
    p1,p2,p3,rot,psi = parametros

    if (min([p1,p2,p3])>=0.0) & (psi>=0) & (psi<np.pi) & (np.abs(rot)<(0.5)):
        a = 0.0
    else:
        a = -np.inf

    b = log_likelihood(parametros,datos,size,intensidades,freqs,fwhms,nlevels)

    return a+b